const typeorm = require('typeorm');
const path = require('path');
const OrmCustomLogger = require('./ormcustomlogger');

//数据库规则 
//1.list数据库中 保存各公司编号的主数据库连接
//2.子数据库名称为 主数据库名称+年
//  子数据库暂时和主数据库在同一个服务器中   以后可以扩展到 只读数据库中
//3.自动建立  子数据库规则 只能建立本年
module.exports = {
    //获取list数据库的数据库连接,可以使用 query执行sql语句
    listdb: function () {
        return global.app.connection;
    },
    listtable: function (tablename) {
        if (!tablename) throw new Error('参数表名不能为空!');
        return global.app.connection.getRepository(tablename).createQueryBuilder();
    },
    maindb: async function (gsbh) {
        if (!gsbh) throw new Error('参数公司编号不能为空!');
        //查询目录数据
        let maincon = await global.app.connection.getRepository('s_公司表').createQueryBuilder()
            .where('I_公司编号=:gsbh', { 'gsbh': gsbh }).getOne();
        if (!maincon) throw new Error('公司编号(' + gsbh + ')不存在');
        return await getcon({
            'host': maincon.C_数据库配置.host,
            'port': 3306,
            'username': maincon.C_数据库配置.username,
            'password': maincon.C_数据库配置.password,
            'database': maincon.C_数据库配置.database
        });
    },
    maintable: async function (gsbh, tablename) {
        if (!tablename) throw new Error('参数表名不能为空!');
        if (!gsbh) throw new Error('参数公司编号不能为空!');
        let con = await this.maindb(gsbh);
        return con.getRepository(tablename).createQueryBuilder();
    },
    subdb: async function (gsbh, year) {
        if (!gsbh) throw new Error('参数公司编号不能为空!');
        year = year || (new Date().getFullYear().toString());
        //查询目录数据
        let maincon = await global.app.connection.getRepository('s_公司表').createQueryBuilder()
            .where('I_公司编号=:gsbh', { 'gsbh': gsbh }).getOne();
        if (!maincon) throw new Error('公司编号(' + gsbh + ')不存在');
        return await getcon({
            'host': maincon.C_数据库配置.host,
            'port': 3306,
            'username': maincon.C_数据库配置.username,
            'password': maincon.C_数据库配置.password,
            'database': maincon.C_数据库配置.database + '_' + year
        }, 'issub');
    },
    subtable: async function (gsbh, tablename, year) {
        if (!gsbh) throw new Error('参数公司编号不能为空!');
        if (!tablename) throw new Error('参数表名不能为空!');
        let con = await this.subdb(gsbh, year);
        return con.getRepository(tablename).createQueryBuilder();
    },
};

async function getcon(ConOpt, issub) {
    let ConnectionManager = typeorm.getConnectionManager();
    let subCon = null;
    let conname = ConOpt.host + ConOpt.database;

    if (ConnectionManager.has(conname)) {
        subCon = ConnectionManager.get(conname);
    }
    else {
        //只有主数据库和本年度的子数据库才能自动创建
        if (!issub || (issub && ConOpt.database.endsWith(new Date().getFullYear().toString()))) {
            let SCHEMATA = await global.app.connection.query('select SCHEMA_NAME from information_schema.SCHEMATA where SCHEMA_NAME="' + ConOpt.database + '"');
            if (SCHEMATA.length == 0) {
                await global.app.connection.query('CREATE DATABASE IF NOT EXISTS ' + ConOpt.database + ' DEFAULT CHARSET utf8 COLLATE utf8_general_ci;');
                //await app.connection.query('insert into mysql.db (`Host`,`Db`,`User`,`Select_priv`,`Insert_priv`,`Update_priv`,`Delete_priv`,`Create_priv`,`Drop_priv`,`Grant_priv`,`References_priv`,`Index_priv`,`Alter_priv`,`Create_tmp_table_priv`,`Lock_tables_priv`,`Create_view_priv`,`Show_view_priv`,`Create_routine_priv`,`Alter_routine_priv`,`Execute_priv`) values ('' + ConOpt.host + '','' + ConOpt.database + '','' + ConOpt.username + '','Y','Y','Y','Y','Y','Y','N','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y')');
            }
        }

        subCon = ConnectionManager.create({
            'name': conname,
            'type': 'mysql',
            'host': ConOpt.host,
            'port': ConOpt.port,
            'username': ConOpt.username,
            'password': ConOpt.password,
            'database': ConOpt.database,
            'synchronize': false,
            'entitySchemas': [
                path.join(__dirname, '../../entity/' + (issub ? 'subDB' : 'mainDB') + '/*.json')
            ],
            'logging': 'all',  //['query', 'error'],
            logger: OrmCustomLogger
        });
        await subCon.connect();
    }
    return subCon;
}